package com.yizhi.lecturer.application.feign;


import com.yizhi.lecturer.application.vo.*;
import com.yizhi.lecturer.application.vo.domain.LecturerClassificationVo;
import com.yizhi.lecturer.application.vo.domain.LecturerGradeVo;
import com.yizhi.lecturer.application.vo.domain.LecturerResponseVo;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author lilingye
 * @date 2019-11-7
 */
@FeignClient(name = "lecturer", contextId = "LecturerClassificationClient")
public interface LecturerClassificationClient {
    /**
     * 获分类列表
     *
     * @param type
     * @return
     */
    @GetMapping(value = "/classification/list")
    List<CommonTree> getClassificationList(@RequestParam(name = "type", defaultValue = "1",required = false) Integer type);

    /**
     * 新增分类
     *
     * @param lecturerClassificationVO
     * @return
     */
    @PostMapping(value = "/classification/add")
    LecturerClassificationVO addClassification(@RequestBody LecturerClassificationVO lecturerClassificationVO);

    /**
     * 修改分类
     *
     * @param lecturerClassificationVO
     * @return
     */
    @PostMapping(value = "/classification/update")
    LecturerClassificationVO updateClassification(@RequestBody LecturerClassificationVO lecturerClassificationVO);

    /**
     * 上下架
     * @param lecturerClassificationVO
     * @return
     */
    @PostMapping(value = "/classification/upOrDown")
    LecturerClassificationRespVO upOrDown(@RequestBody LecturerClassificationVO lecturerClassificationVO);

    /**
     * 删除分类
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/classification/del")
    LecturerResponseVo del(@RequestParam(name="id") Long id);


    @PostMapping(value = "/classification/sort")
    Boolean sort(@RequestBody LecturerSortVO lecturerSortVO);


    /**
     * 获取分类详情
     * @param id
     * @return
     */
    @GetMapping(value = "/classification/get/id")
    LecturerClassificationVo getById(@RequestParam(name = "id") Long id);

    /**
     * 新增讲师评级
     *
     * @param lecturerGradeVO
     * @return
     */
    @PostMapping(value = "/grade/add")
    LecturerGradeVO add(@RequestBody LecturerGradeVO lecturerGradeVO);

    /**
     * 修改，删除，上下架
     *
     * @param lecturerGradeVO
     * @return
     */
    @PostMapping(value = "/grade/update")
    LecturerGradeVO update(@RequestBody LecturerGradeVO lecturerGradeVO);

    /**
     * 根据站点获取江湖级别列表
     *
     * @return
     */
    @GetMapping(value = "/grade/list")
    List<LecturerGradeVo> list(@RequestParam(value = "status") Integer status);

}
