package com.yizhi.application.Controller;


import com.yizhi.application.domain.LecturerClassification;
import com.yizhi.application.domain.LecturerResponse;
import com.yizhi.application.service.ILecturerClassificationService;
import com.yizhi.lecturer.application.vo.LecturerClassificationRespVO;
import com.yizhi.lecturer.application.vo.LecturerClassificationVO;
import com.yizhi.lecturer.application.vo.LecturerSortVO;
import com.yizhi.util.application.tree.CommonTree;
import io.swagger.annotations.Api;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * <p>
 * 讲师库分类表 前端控制器
 * </p>
 *
 * @author lilingye
 * @since 2019-09-09
 */
@Api(value = "讲师库分类", tags = "讲师库分类")
@RestController
@RequestMapping("/classification")
public class LecturerClassificationController {
    @Autowired
    private ILecturerClassificationService classificationService;

    /**
     * 分类列表
     *
     * @param type
     * @return
     */
    @GetMapping(value = "/list")
    public List<CommonTree> getClassificationList(@RequestParam(name = "type", defaultValue = "1",required = false) Integer type) {
        return classificationService.getClassificationList(type);
    }

    /**
     * 新增分类
     *
     * @param lecturerClassificationVO
     * @return
     */
    @PostMapping(value = "/add")
    public LecturerClassificationVO addClassification(@RequestBody LecturerClassificationVO lecturerClassificationVO) {
        return classificationService.add(lecturerClassificationVO);
    }

    /**
     * 修改分类
     *
     * @param lecturerClassificationVO
     * @return
     */
    @PostMapping(value = "/update")
    public LecturerClassificationVO updateClassification(@RequestBody LecturerClassificationVO lecturerClassificationVO) {
        return classificationService.update(lecturerClassificationVO);
    }

    /**
     * 上下架分类
     * @param lecturerClassificationVO
     * @return
     */
    @PostMapping(value = "/upOrDown")
    public LecturerClassificationRespVO upOrDown(@RequestBody LecturerClassificationVO lecturerClassificationVO) {
        return classificationService.upOrDown(lecturerClassificationVO);
    }


    /**
     * 删除分类
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/del")
    public LecturerResponse del(@RequestParam(name = "id") Long id) {
        return classificationService.del(id);
    }

    /**
     * 分类排序
     *
     * @param lecturerSortVO
     * @return
     */
    @PostMapping(value = "/sort")
    public Boolean sort(@RequestBody LecturerSortVO lecturerSortVO) {
        return classificationService.sortClassification(lecturerSortVO);
    }

    /**
     * 获取实体
     * @param id
     * @return
     */
    @GetMapping(value = "/get/id")
    public LecturerClassification getById(@RequestParam(name = "id") Long id) {
        return classificationService.selectById(id);
    }
}

