package com.yizhi.application.mapper;

import com.baomidou.mybatisplus.mapper.BaseMapper;
import com.yizhi.application.domain.LecturerClassification;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;

/**
 * <p>
 * 讲师库分类表 Mapper 接口
 * </p>
 *
 * @author lingye
 * @since 2019-09-09
 */
public interface LecturerClassificationMapper extends BaseMapper<LecturerClassification> {
    /**
     * 根据站点返回排序
     *
     * @param siteId
     * @return
     */
    @Select("select IFNULL(max(sort),0) from lecturer_classification where site_id =#{siteId} and parent_id =#{parentId}")
    int queryClassificationMaxSort(@Param("siteId") Long siteId, @Param("parentId") Long parentId);

    /**
     * 查询前一个讲师库分类的sort
     *
     * @param siteId
     * @param sort
     * @return
     */
    LecturerClassification getClassificationBeforeSort(@Param("siteId") Long siteId, @Param("sort") int sort, @Param("parentId") Long parentId);

    /**
     * 查询后一个讲师库分类的sort
     *
     * @param siteId
     * @param sort
     * @return
     */
    LecturerClassification getClassificationAfterSort(@Param("siteId") Long siteId, @Param("sort") int sort, @Param("parentId") Long parentId);


    /**
     * 查询列表
     *
     * @param siteId
     * @param sort
     * @return
     */
    List<LecturerClassification> getClassificationBeforeSortList(@Param("siteId") Long siteId, @Param("sort") int sort);

    /**
     * 查询分类列表
     * @param id
     * @return
     */
    List<LecturerClassification> selectClassificationList(@Param("id") Long id);

    /**
     * 学员端接口
     * @param siteId 站点id
     * @param id 分类id
     * @return
     */
    List<LecturerClassification> selectStuClassificationList(@Param("siteId") Long siteId,@Param("id") Long id);

    /**
     * 获取所有的分类
     * @param siteId
     * @return
     */
    List<LecturerClassification> selectStuParentIds(@Param("siteId") Long siteId);

}
