package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.domain.Lecturer;
import com.yizhi.application.domain.LecturerClassification;
import com.yizhi.application.domain.LecturerResponse;
import com.yizhi.application.domain.TrLecturerClassification;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.lecturer.application.enums.DeleteFlag;
import com.yizhi.lecturer.application.enums.LecturerEnum;
import com.yizhi.application.mapper.LecturerClassificationMapper;
import com.yizhi.application.service.ILecturerClassificationService;
import com.yizhi.application.service.ITrLecturerClassificationService;
import com.yizhi.lecturer.application.vo.LecturerClassificationRespVO;
import com.yizhi.lecturer.application.vo.LecturerClassificationVO;
import com.yizhi.lecturer.application.vo.LecturerSortVO;
import com.yizhi.util.application.tree.CommonTree;
import com.yizhi.util.application.tree.TreeUtil;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static java.util.stream.Collectors.toList;

/**
 * <p>
 * 讲师库分类表 服务实现类
 * </p>
 *
 * @author lingye
 * @since 2019-09-09
 */
@Service
public class LecturerClassificationServiceImpl extends ServiceImpl<LecturerClassificationMapper, LecturerClassification> implements ILecturerClassificationService {
    private static final Logger LOGGER = LoggerFactory.getLogger(LecturerClassificationServiceImpl.class);

    @Autowired
    IdGenerator idGenerator;

    @Autowired
    ITrLecturerClassificationService trLecturerClassificationService;

    @Override
    public LecturerClassificationVO add(LecturerClassificationVO lecturerClassificationVO) {
        RequestContext context = ContextHolder.get();
        Long id = idGenerator.generate();
        Integer layer = 0;
        LecturerClassification lecturerClassification = new LecturerClassification();
        BeanUtils.copyProperties(lecturerClassificationVO, lecturerClassification);
        if ((DeleteFlag.NO.ordinal()) != lecturerClassificationVO.getParentId()) {
            LOGGER.info("非父级类");
            LecturerClassification classification = new LecturerClassification();
            classification.setDelFlag(DeleteFlag.NO.ordinal());
            classification.setId(lecturerClassificationVO.getParentId());
            LecturerClassification c = this.baseMapper.selectOne(classification);
            layer = c.getLayer() + DeleteFlag.YES.toValue();
        } else {
            lecturerClassification.setStatus(DeleteFlag.YES.ordinal());
            LOGGER.info("实体：{}", lecturerClassificationVO);
        }

        lecturerClassification.setId(id);
        lecturerClassification.setCompanyId(context.getCompanyId());
        lecturerClassification.setSiteId(context.getSiteId());
        lecturerClassification.setOrgId(context.getOrgId());
        lecturerClassification.setCreateById(context.getAccountId());
        lecturerClassification.setUpdateById(context.getAccountId());
        lecturerClassification.setLayer(layer);

        lecturerClassification.setCreateTime(new Date());
        lecturerClassification.setUpdateTime(new Date());
        this.classificationSetParentIds(lecturerClassification);
        int lastSort = this.baseMapper.queryClassificationMaxSort(context.getSiteId(), lecturerClassification.getParentId());
        lecturerClassification.setSort(lastSort + DeleteFlag.YES.toValue());

        this.baseMapper.insert(lecturerClassification);
        BeanUtils.copyProperties(lecturerClassification, lecturerClassificationVO);
        return lecturerClassificationVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public LecturerClassificationVO update(LecturerClassificationVO lecturerClassificationVO) {

        RequestContext context = ContextHolder.get();
        LecturerClassification lecturerClassification = new LecturerClassification();
        BeanUtils.copyProperties(lecturerClassificationVO, lecturerClassification);

        lecturerClassification.setCompanyId(context.getCompanyId());
        lecturerClassification.setSiteId(context.getSiteId());
        lecturerClassification.setCreateById(context.getAccountId());
        lecturerClassification.setUpdateTime(new Date());
        lecturerClassification.setStatus(null);

        boolean flag = this.updateById(lecturerClassification);
        LOGGER.info("修改成功：" + flag);
        BeanUtils.copyProperties(lecturerClassification, lecturerClassificationVO);

        return lecturerClassificationVO;
    }

    /**
     * 分类的上架和下架
     *
     * @param lecturerClassificationVO
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public LecturerClassificationRespVO upOrDown(LecturerClassificationVO lecturerClassificationVO) {

        LecturerClassification lecturerClassification = new LecturerClassification();
        BeanUtils.copyProperties(lecturerClassificationVO, lecturerClassification);
        LecturerClassificationRespVO lecturerClassificationRespVO = new LecturerClassificationRespVO();


        //下架下面所有的分类
        LecturerClassification lc = null;
        if (lecturerClassificationVO.getStatus() == DeleteFlag.NO.ordinal()) {



            this.updateById(lecturerClassification);
            lc = new LecturerClassification();
            lc.setParentId(lecturerClassificationVO.getId());
            lc.setDelFlag(DeleteFlag.NO.ordinal());
            // 子分类
            List<LecturerClassification> lcOnes = this.selectList(QueryUtil.condition(lc));
            if (CollectionUtils.isNotEmpty(lcOnes)) {

                List<LecturerClassification> lecturerClassifications = this.baseMapper.selectClassificationList(lecturerClassificationVO.getId());
                lecturerClassifications = lecturerClassifications.parallelStream().map(lcs->{
                    lcs.setStatus(DeleteFlag.NO.ordinal());
                    return lcs;
                }).collect(toList());
                this.updateBatchById(lecturerClassifications);

            } else {
                this.updateById(lecturerClassification);
            }
            lecturerClassificationRespVO.setCode(1);

        } else if (lecturerClassificationVO.getStatus() == DeleteFlag.YES.ordinal()) {
            LecturerClassification le = this.selectById(lecturerClassificationVO.getId());
            le = this.selectById(le.getParentId());
            Integer status = 0;
            if (le != null) {
                status = le.getStatus();
                if (status == DeleteFlag.NO.ordinal()) {
                    lecturerClassificationRespVO.setCode(DeleteFlag.NO.ordinal());
                } else {
                    boolean flag = this.updateById(lecturerClassification);
                    lecturerClassificationRespVO.setCode(DeleteFlag.YES.ordinal());
                    LOGGER.info("操作结果：" + flag);
                }
            } else {
                boolean flag = this.updateById(lecturerClassification);
                lecturerClassificationRespVO.setCode(DeleteFlag.YES.ordinal());
                LOGGER.info("操作结果：" + flag);
            }

        }
        return lecturerClassificationRespVO;
    }


    @Override
    public boolean sortClassification(LecturerSortVO lecturerSortVO) {
        RequestContext context = ContextHolder.get();
        String code = lecturerSortVO.getCode();
        Long id = lecturerSortVO.getId();

        LecturerClassification currentLecturer = this.selectById(id);
        int currentSort = currentLecturer.getSort();
        Long parentId = currentLecturer.getParentId();
        if (code.equals(LecturerEnum.UP.getKey())) {
            //上移
            LecturerClassification beforeSortLecturer = this.baseMapper.getClassificationBeforeSort(context.getSiteId(), currentSort, parentId);
            if (beforeSortLecturer != null) {
                int beforeSort = beforeSortLecturer.getSort();

                beforeSortLecturer.setSort(currentSort);
                this.updateById(beforeSortLecturer);

                currentLecturer.setSort(beforeSort);
                this.updateById(currentLecturer);
            }


        } else if (code.equals(LecturerEnum.DOWN.getKey())) {
            //下移
            LecturerClassification beforeSortLecturer = this.baseMapper.getClassificationAfterSort(context.getSiteId(), currentSort, parentId);
            if (beforeSortLecturer != null) {
                int beforeSort = beforeSortLecturer.getSort();
                if (beforeSort != DeleteFlag.NO.ordinal()) {
                    beforeSortLecturer.setSort(currentSort);
                    this.updateById(beforeSortLecturer);

                    currentLecturer.setSort(beforeSort);
                    this.updateById(currentLecturer);
                }
            }
        } else if (code.equals(LecturerEnum.TOP.getKey())) {
            //置顶
            List<LecturerClassification> lecturerBeforeSortList = this.baseMapper.getClassificationBeforeSortList(context.getSiteId(), currentSort);

            lecturerBeforeSortList = lecturerBeforeSortList.parallelStream().map(lecturer -> {
                lecturer.setSort(lecturer.getSort() + DeleteFlag.YES.ordinal());
                return lecturer;
            }).collect(toList());

            this.updateBatchById(lecturerBeforeSortList);
            Lecturer lecturer = new Lecturer();
            lecturer.setId(id);
            LecturerClassification updateLecturer = this.selectById(lecturer);
            updateLecturer.setSort(DeleteFlag.NO.ordinal());
            this.updateById(updateLecturer);

        }

        return true;
    }

    /**
     * 父级i的保存
     *
     * @param classification
     */

    private void classificationSetParentIds(LecturerClassification classification) {

        if (classification.getParentId()==null || classification.getParentId().equals(0L)) {
            classification.setParentId(0L);
            classification.setParentIds("0,");
        } else {
            Long pid = classification.getParentId();
            LecturerClassification temp = this.selectById(pid);
            String pids = temp.getParentIds();
            classification.setParentId(pid);
            classification.setParentIds(pids + pid + ",");
        }
    }


    @Override
    public List<CommonTree> getClassificationList(Integer type) {
        RequestContext context = ContextHolder.get();
        HQueryUtil.startHQ(LecturerClassification.class);
        List<LecturerClassification> lecturerClassificationList = getClassificationList(context.getCompanyId(), context.getSiteId(), type);
        if (CollectionUtils.isNotEmpty(lecturerClassificationList)) {
            List<CommonTree> orgTreeList = new ArrayList<>(lecturerClassificationList.size());
            for (LecturerClassification classification : lecturerClassificationList) {
                CommonTree commonTree = new CommonTree();
                commonTree.setPId(classification.getParentId());
                commonTree.setId(classification.getId());
                commonTree.setName(classification.getName());
                commonTree.setNode(classification);
                orgTreeList.add(commonTree);
            }
            orgTreeList = TreeUtil.buildTree(orgTreeList);
            return orgTreeList;
        }
        return null;
    }

    /**
     * 删除讲师分类
     *
     * @param id
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public LecturerResponse del(Long id) {
        LecturerResponse lecturerResponse = new LecturerResponse();
        if (!trLecturerClassificationService.getRelateLecturer(id)) {
            LecturerClassification classification = new LecturerClassification();
            classification.setDelFlag(0);
            classification.setParentId(id);
            int count = this.selectCount(QueryUtil.condition(classification));

            if (count > 0) {
                //存在子类不能删除
                lecturerResponse.setCode("fail");
                lecturerResponse.setMsg("请删除子类后，再进行删除");
                return lecturerResponse;
            } else {
                lecturerResponse.setCode("success");
                classification.setDelFlag(DeleteFlag.YES.ordinal());
                classification.setParentId(null);
                classification.setId(id);
                TrLecturerClassification tlc = new TrLecturerClassification();
                tlc.setClassificationId(id);
                List<TrLecturerClassification> trLecturerClassifications = trLecturerClassificationService.selectList(QueryUtil.condition(tlc));
                LOGGER.info("返回的列表："+trLecturerClassifications);
                this.updateById(classification);

                if (CollectionUtils.isNotEmpty(trLecturerClassifications)){
//                    trLecturerClassificationService.deleteBatchIds(trLecturerClassifications);
                    for (TrLecturerClassification tr: trLecturerClassifications){
                        trLecturerClassificationService.deleteById(tr.getId());
                    }
                }
                return lecturerResponse;
            }
        } else {
            lecturerResponse.setCode("fail");
            lecturerResponse.setMsg("该分类已被讲师使用，无法删除！");
            return lecturerResponse;
        }


    }

    private List<LecturerClassification> getClassificationList(Long companyId, Long siteId, Integer type) {
        LecturerClassification classification = new LecturerClassification();
        classification.setDelFlag(DeleteFlag.NO.ordinal());
        classification.setCompanyId(companyId);
        classification.setSiteId(siteId);
        if (type == DeleteFlag.YES.ordinal()) {
            //type 为0 的时候显示上架的分类
            LOGGER.info("管理端的参数");
            EntityWrapper<LecturerClassification> condition = QueryUtil.condition(classification);
            condition.orderBy("sort", true);
            return this.baseMapper.selectList(condition);
        } else {
            List<LecturerClassification> lecturerClassifications = new ArrayList<>();
            List<LecturerClassification> ids = this.baseMapper.selectStuParentIds(siteId);
            if (CollectionUtils.isNotEmpty(ids)) {
                for (LecturerClassification lct:ids){
                    lecturerClassifications.add(lct);
                    List<LecturerClassification> lecturerClassificationList = this.baseMapper.selectStuClassificationList(siteId, lct.getId());
                    lecturerClassifications.addAll(lecturerClassificationList);
                }
                return lecturerClassifications;
            } else {
                return new ArrayList<>();
            }
        }
    }
}
