package com.yizhi.application.service.impl;

import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.application.domain.LecturerInfo;
import com.yizhi.application.mapper.LecturerInfoMapper;
import com.yizhi.application.orm.util.QueryUtil;
import com.yizhi.application.service.ILecturerInfoService;
import com.yizhi.lecturer.application.vo.LecturerInfoVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lingye
 * @since 2019-11-06
 */
@Service
public class LecturerInfoServiceImpl extends ServiceImpl<LecturerInfoMapper, LecturerInfo> implements ILecturerInfoService {

    @Autowired
    IdGenerator idGenerator;

    /**
     * 基本信息新增
     *
     * @param lecturerInfoVO
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public LecturerInfoVO add(LecturerInfoVO lecturerInfoVO) {

        LecturerInfo lecturerInfo = new LecturerInfo();
        RequestContext context = ContextHolder.get();
        lecturerInfo.setSiteId(context.getSiteId());
        // 先删除配置信息
        this.delete(QueryUtil.condition(lecturerInfo));

        BeanUtils.copyProperties(lecturerInfoVO, lecturerInfo);
        lecturerInfo.setId(idGenerator.generate());
        lecturerInfo.setCreateById(context.getAccountId());
        lecturerInfo.setCompanyId(context.getCompanyId());
        this.insert(lecturerInfo);

        BeanUtils.copyProperties(lecturerInfo, lecturerInfoVO);
        return lecturerInfoVO;
    }

    /**
     * 修改
     *
     * @param lecturerInfoVO
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public LecturerInfoVO update(LecturerInfoVO lecturerInfoVO) {
        LecturerInfo lecturerInfo = new LecturerInfo();
        BeanUtils.copyProperties(lecturerInfoVO, lecturerInfo);
        RequestContext context = ContextHolder.get();
        lecturerInfo.setCreateById(context.getAccountId());
        lecturerInfo.setSiteId(context.getSiteId());
        lecturerInfo.setCompanyId(context.getCompanyId());
        this.updateById(lecturerInfo);
        BeanUtils.copyProperties(lecturerInfo, lecturerInfoVO);
        return lecturerInfoVO;
    }

    /**
     * 根据站点获取讲师基本信息
     *
     * @return
     */
    @Override
    public LecturerInfoVO getBySiteId() {

        LecturerInfoVO lecturerInfoVO = new LecturerInfoVO();
        RequestContext context = ContextHolder.get();
        LecturerInfo lecturerInfo = new LecturerInfo();
        lecturerInfo.setSiteId(context.getSiteId());
        LecturerInfo info = this.selectOne(QueryUtil.condition(lecturerInfo));
        if (info != null) {
            BeanUtils.copyProperties(info, lecturerInfoVO);
            return lecturerInfoVO;
        } else {
            return null;
        }
    }
}
